@@ -1,5 +1,5 @@
-// edition: 2021
-// check-pass
+//@ edition: 2021
+//@ check-pass
 
 #![feature(return_type_notation)]
 //~^ WARN the feature `return_type_notation` is incomplete
@@ -10,7 +10,7 @@ trait HealthCheck {
 
 async fn do_health_check_par<HC>(hc: HC)
 where
-    HC: HealthCheck<check(): Send> + Send + 'static,
+    HC: HealthCheck<check(..): Send> + Send + 'static,
 {
     spawn(async move {
         let mut hc = hc;
